Index: bp-xprofile/bp-xprofile-screens.php
===================================================================
--- bp-xprofile/bp-xprofile-screens.php	(revision 4501)
+++ bp-xprofile/bp-xprofile-screens.php	(working copy)
@@ -28,7 +28,7 @@
  * @uses bp_core_load_template() Looks for and loads a template file within the current member theme (folder/filename)
  */
 function xprofile_screen_edit_profile() {
-	global $bp;
+	global $bp, $wp_query;
 
 	if ( !bp_is_my_profile() && !is_super_admin() )
 		return false;
@@ -38,8 +38,12 @@
 		bp_core_redirect( $bp->displayed_user->domain . $bp->profile->slug . '/edit/group/1' );
 
 	// Check the field group exists
-	if ( !xprofile_get_field_group( $bp->action_variables[1] ) )
-		bp_core_redirect( bp_get_root_domain() );
+	if ( !xprofile_get_field_group( $bp->action_variables[1] ) ) {
+		$wp_query->set_404();
+		status_header( 404 );
+		nocache_headers();
+		return;
+	}
 
 	// Check to see if any new information has been submitted
 	if ( isset( $_POST['field_ids'] ) ) {
Index: bp-activity/bp-activity-screens.php
===================================================================
--- bp-activity/bp-activity-screens.php	(revision 4501)
+++ bp-activity/bp-activity-screens.php	(working copy)
@@ -83,7 +83,7 @@
 add_action( 'bp_activity_screen_mentions', 'bp_activity_reset_my_new_mentions' );
 
 function bp_activity_screen_single_activity_permalink() {
-	global $bp;
+	global $bp, $wp_query;
 
 	// No displayed user or not viewing activity component
 	if ( !bp_is_activity_component() )
@@ -95,8 +95,13 @@
 	// Get the activity details
 	$activity = bp_activity_get_specific( array( 'activity_ids' => bp_current_action() ) );
 
-	if ( !$activity = $activity['activities'][0] )
-		bp_core_redirect( bp_get_root_domain() );
+	// 404 if activity does not exist
+	if ( !$activity = $activity['activities'][0] ) {
+		$wp_query->set_404();
+		status_header( 404 );
+		nocache_headers();
+		return;
+	}
 
 	// Default access is true
 	$has_access = true;
@@ -105,8 +110,12 @@
 	if ( isset( $bp->groups->id ) && $activity->component == $bp->groups->id ) {
 
 		// Activity is from a group, but groups is currently disabled
-		if ( !bp_is_active( 'groups') )
-			bp_core_redirect( bp_get_root_domain() );
+		if ( !bp_is_active( 'groups') ) {
+			$wp_query->set_404();
+			status_header( 404 );
+			nocache_headers();
+			return;
+		}
 
 		// Check to see if the group is not public, if so, check the
 		// user has access to see this activity

Index: bp-core/bp-core-catchuri.php
===================================================================
--- bp-core/bp-core-catchuri.php	(revision 4501)
+++ bp-core/bp-core-catchuri.php	(working copy)
@@ -351,32 +351,30 @@
  * @since 1.3
  */
 function bp_core_catch_no_access() {
-	global $bp, $wp_query, $bp_unfiltered_uri, $bp_no_status_set;
+	global $bp, $bp_no_status_set, $wp_query;
 
 	// If bp_core_redirect() and $bp_no_status_set is true,
 	// we are redirecting to an accessible page, so skip this check.
 	if ( $bp_no_status_set )
 		return false;
 
 	// If the displayed user was marked as a spammer and the logged-in user is not a super admin, redirect
 	if ( isset( $bp->displayed_user->id ) && bp_core_is_user_spammer( $bp->displayed_user->id ) ) {
-		if ( !is_super_admin() )
-			bp_core_redirect( $bp->root_domain );
-		else
+		if ( !$bp->loggedin_user->is_super_admin ) {
+			$wp_query->set_404();
+			status_header( 404 );
+			nocache_headers();
+			return;
+
+		} else {
 			bp_core_add_message( __( 'This user has been marked as a spammer. Only site admins can view this profile.', 'buddypress' ), 'error' );
+		}
 	}
 
-	// If BP_ENABLE_ROOT_PROFILES is not defined and the displayed user does not exist, redirect
-	if ( !$bp->displayed_user->id && isset( $bp_unfiltered_uri[0] ) && $bp_unfiltered_uri[0] == $bp->members->slug && isset( $bp_unfiltered_uri[1] ) )
-		bp_core_redirect( $bp->root_domain );
-
-	// Access control!
 	if ( !isset( $wp_query->queried_object ) && !bp_is_blog_page() ) {
-		if ( is_user_logged_in() ) {
-			bp_core_no_access( array( 'redirect' => false, 'message' => __( 'You do not have access to that page', 'buddypress' ) ) );
-		} else {
-			bp_core_no_access();
-		}
+		$wp_query->set_404();
+		status_header( 404 );
+		nocache_headers();
 	}
 }
 add_action( 'wp', 'bp_core_catch_no_access' );
@@ -402,15 +400,6 @@
 	$r = wp_parse_args( $args, $defaults );
 	extract( $r, EXTR_SKIP );
 
-	// Group filtering
-	// When a user doesn't have access to a group's activity / secondary page, redirect to group's homepage
-	if ( !$redirect ) {
-		if ( bp_is_active( 'groups' ) && bp_is_current_component( 'groups' ) ) {
-			$root = bp_get_group_permalink( $bp->groups->current_group );
-			$message = false;
-		}
-	}
-
 	// Apply filters to these variables
 	$mode		= apply_filters( 'bp_no_access_mode', $mode, $root, $redirect, $message );
 	$redirect	= apply_filters( 'bp_no_access_redirect', $redirect, $root, $message, $mode );
@@ -351,32 +351,30 @@
 		return false;
 
-	// If the displayed user was marked as a spammer and the logged-in user is not a super admin, redirect
+	// If the displayed user was marked as a spammer and the logged-in user is not a super admin, 404.
 	if ( isset( $bp->displayed_user->id ) && bp_core_is_user_spammer( $bp->displayed_user->id ) ) {
Index: bp-activity/bp-activity-actions.php
===================================================================
--- bp-activity/bp-activity-actions.php	(revision 4501)
+++ bp-activity/bp-activity-actions.php	(working copy)
@@ -13,7 +13,7 @@
 add_action( 'bp_init', 'bp_register_activity_actions', 8 );
 
 function bp_activity_action_permalink_router() {
-	global $bp;
+	global $bp, $wp_query;
 
 	// Not viewing activity
 	if ( ( $bp->activity->slug != bp_current_component() ) || !bp_is_current_action( 'p' ) )
@@ -26,9 +26,13 @@
 	// Get the activity details
 	$activity = bp_activity_get_specific( array( 'activity_ids' => $bp->action_variables[0] ) );
 
-	// Redirect to root if activity does not exist
-	if ( !$activity = $activity['activities'][0] )
-		bp_core_redirect( bp_get_root_domain() );
+	// 404 if activity does not exist
+	if ( !$activity = $activity['activities'][0] ) {
+		$wp_query->set_404();
+		status_header( 404 );
+		nocache_headers();
+		return;
+	}
 
 	// Do not redirect at default
 	$redirect = false;